// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "scene/SceneItemComponent.hpp"
#include "scene/components/physics/2d/CharacterController2D.hpp"

namespace Dawn {
  struct DamageInformation {
    int32_t amount = 0;
    float_t invincibleTime = 2.0f;
    float_t stunTime = 0.4f;
  };

  class EntityHealth : public SceneItemComponent {
    public:
      // @optional
      int32_t health = 10;
      // @optional
      int32_t maxHealth = 10;
      // @optional
      float_t invincibleTime = 0.0f;
      // @optional
      float_t stunTime = 0.0f;
      
      EntityHealth(SceneItem* item);
      void onStart() override;

      /**
       * Returns true if the entity is invincible
       * 
       * @return True if invincible, false otherwise.
       */
      bool_t isInvincible();

      /**
       * Returns true if the entity is stunned.
       * 
       * @return True if stunned, false otherwise.
       */
      bool_t isStunned();

      /**
       * Apply damage to this entity.
       * 
       * @param info The damage information.
       */
      void damage(struct DamageInformation info);
  };
}